﻿-- D'Artagnan v0.2.3
-- Report manager for hunting on bots
-- Written by OUGHT
-- Idea by Overpro

local addonName, addon = ...
addon.version = "0.2.3";
local L = addon;
addon.GuiShown = false;
addon.HelpShown = false;
addon.delta = 0;
local AceGUI = LibStub("AceGUI-3.0");
local AceConfig = LibStub("AceConfig-3.0");
local AceConfigDialog = LibStub("AceConfigDialog-3.0");

local defaults = {
	["bgChatReport"] = false,
	["bgReportAFK"] = true,
	["bgHookAFK"] = true,
	["rpAllowDouble"] = false,
	["bgRemoveByWhisper"] = true,
};

local reports = {};

local options = {
	type = "group",
	args = {
		btnHelp = {
			name  = L["HelpName"],
			desc  = L["HelpDesc"],
			type  = "execute",
			order = 0;
			width = "full";
			func  = function() addon.ShowHelp() end
		},
		bgChatReport = {
			name  = L["BGChatReportName"],
			desc  = L["BGChatReportDesc"],
			type  = "toggle",
			order = 1;
			width = "full";
			set   = function(i, v) addon.db.bgChatReport = v; end,
			get   = function(i) return addon.db.bgChatReport; end,
		},
		-- Blizzard made function ReportPlayerIsPVPAFK secure. They want to enlarge population of bots???
		--[[bgReportAFK = {
			name  = L["BGReportAFKName"],
			desc  = L["BGReportAFKDesc"],
			type  = "toggle",
			order = 2;
			width = "full";
			set   = function(i, v) addon.db.bgReportAFK = v; end,
			get   = function(i) return addon.db.bgReportAFK; end,
		},]]
		bgHookAFK = {
			name  = L["BGHookAFKName"],
			desc  = L["BGHookAFKDesc"],
			type  = "toggle",
			order = 3;
			width = "full";
			set   = function(i, v) addon.db.bgHookAFK = v; end,
			get   = function(i) return addon.db.bgHookAFK; end,
		},
		bgRemoveByWhisper = {
			name  = L["BGRemoveByWhisperName"],
			desc  = L["BGRemoveByWhisperDesc"],
			type  = "toggle",
			order = 4;
			width = "full";
			set   = function(i, v) addon.db.bgRemoveByWhisper = v; end,
			get   = function(i) return addon.db.bgRemoveByWhisper; end,
		},
		rpAllowDouble = {
			name  = L["RPAllowDoubleName"],
			desc  = L["RPAllowDoubleDesc"],
			type  = "toggle",
			order = 5;
			width = "full";
			set   = function(i, v) addon.db.rpAllowDouble = v; end,
			get   = function(i) return addon.db.rpAllowDouble; end,
		},
	}
};

SLASH_DARTAGNAN1 = "/dartagnan";
SLASH_DARTAGNAN2 = "/d'artagnan";
SLASH_DARTAGNAN3 = "/da";
SlashCmdList["DARTAGNAN"] = function(msg)
	addon.HandleSlash(msg);
end

addon.f = CreateFrame("Frame", addonName.."mainFrame", UIParent);


addon.f:SetScript("OnEvent", function(self, event, ...) 
	  if addon[event] then 
		 return addon[event](addon, event, ...) ;
	  end 
end)

addon.f:RegisterEvent("ADDON_LOADED");
addon.f:RegisterEvent("CHAT_MSG_WHISPER");

function addon:ADDON_LOADED(event, addon)
	if addon == "D'Artagnan" then
		DEFAULT_CHAT_FRAME:AddMessage("|c00FF9900[DA]: |rD'Artagnan v"..self.version.." Loaded");

		if not DArtagnanDB then
			DArtagnanDB = {};
		end
		self.db = self.CopyDefaults(defaults, DArtagnanDB);
		--for i,v in ipairs(self.db) do print(i,v) end
		self.CreateGUI();
		self.CreateOptions();
		
		self.f:UnregisterEvent("ADDON_LOADED");
	end
end

function addon:CHAT_MSG_WHISPER(event, message, sender, language, channelString, target, flags, unknown, channelNumber, channelName, unknown, counter, guid)
	if addon.db.bgRemoveByWhisper == true then
		addon.RemoveByWhisper(sender);
	end
	--print(sender, message);
end

function addon.RemoveByWhisper(sender)
	local n, r = addon.GetNickRealm(sender);
	local idx = nil;
	if r == "" then 
		r = GetRealmName();
	end
	--print(n, r);
	for i = #reports, 1, -1 do
		--print(reports[i].realm, reports[i].name);
		if (reports[i].realm == r) and (reports[i].name == n) then
			idx = i;
			break;
		end
	end
	if idx ~= nil then
		--print(idx);
		local pattern = L["ReportDeleted"]; 
		if #reports == 0 then return end
		local r = reports[idx];
		local res = string.format(pattern, r.name, r.realm);
		print(res);

		if addon.db.bgChatReport == true then
			local msg = string.format(L["BGChatDeleted"], r.name);
			SendChatMessage(msg, "INSTANCE_CHAT")
		end
		table.remove(reports, idx);
	end
end

function addon.GetNickRealm(sender)
	nick, realm = string.match(sender, "([^-]+)%-?([^-]*)")
	return nick, realm;
end

function addon.CopyDefaults(src, dst)
	if not src then return { } end
	if not dst then dst = { } end
	for k, v in pairs(src) do
		if type(v) == "table" then
			dst[k] = addon.CopyDefaults(v, dst[k]);
		elseif type(v) ~= type(dst[k]) then
			dst[k] = v;
		end
	end
	return dst;
end

function addon.HandleSlash(msg)
	if msg == "report" then
		addon.HandleReport("target", true)
	elseif msg == "gui" then
		addon.ShowGUI();
	elseif msg == "cancel" then
		addon.DeleteLastReport();
	else
		addon.ShowHelp();
	end	
end

function addon.HandleReport(unit, gname)	
	local report;
	--print(unit, gname);
	report = addon.MakeReport(unit, gname);

	if report == nil then return end
	if addon.db.rpAllowDouble == false then
		for i, v in ipairs(reports) do
			if (v.name == report.name) and (v.realm == report.realm) then
				local txt = string.format(L["DoubleReport"], report.name, report.realm);
				print(txt);
				return;
			end
		end
	end
	table.insert(reports, report);
	
	local txt = addon.MakeChatReport(report);
	print(txt);

	if addon.db.bgChatReport == true then
		local msg = string.format(L["BGChatReport"], report.name);
		if addon.db.bgRemoveByWhisper == true then
			msg = msg .. L["BGChatWhisper"];
		end
		SendChatMessage(msg, "INSTANCE_CHAT")
	end
end

hooksecurefunc("ReportPlayerIsPVPAFK", function(player)
	if (addon.db.bgHookAFK == true) then
		--print("Reported!", player)
		addon.HandleReport(player, false)	
	end
end)

--[[local OldReportPlayerIsPVPAFK = ReportPlayerIsPVPAFK;
function ReportPlayerIsPVPAFK(unit, ...)
	OldReportPlayerIsPVPAFK(unit);
	--print("Hooked!")
	local n = ...
	if n == nil then n = true end
	addon.HookAFK(unit, n)
end]]

function addon.MakeReport(unit, gname)
	local name, realm;
	if gname == true then
		name, realm = UnitName(unit);
	else
		name, realm = addon.GetNickRealm(unit);
	end
	if name == nil  then return nil end
	if realm == nil then realm = GetRealmName() end
	if not UnitIsPlayer(unit) then return end
	local zone = GetMapNameByID(GetCurrentMapAreaID());

	local time = date("*t")
	local hour, min = GetGameTime()
	local stime = {
		["hour"] = hour,
		["min"]  = min,
	}

	local res = {
		["name"]  = name,
		["realm"] = realm,
		["time"]  = time,
		["stime"] = stime,
		["zone"] = zone,
	}
	return res;
end

function addon.MakeChatReport(r)
	local pattern = L["ChatReport"];
	local res = string.format(pattern, r.name, r.realm, r.stime.hour, r.stime.min, r.time.sec, L["OneForAll"]);
	return res;
end

function addon.MakeTextReport()
	local res = L["Head"];
	local pattern = L["TextReport"];
	for i, r in ipairs(reports) do
		if r.zone == nil then r.zone = "Temp!" end
		res = res .. string.format(pattern, r.time.day, r.time.month, r.time.year, r.stime.hour, r.stime.min, r.time.sec, r.name, r.realm, r.zone);
	end
	return res;
end

function addon.DeleteLastReport(delta)
	if addon.delta ~= delta then
		addon.delta = delta;

		local pattern = L["ReportDeleted"]; 
		if #reports == 0 then return end
		local r = reports[#reports];
		local res = string.format(pattern, r.name, r.realm);
		print(res);

		if addon.db.bgChatReport == true then
			local msg = string.format(L["BGChatDeleted"], r.name);
			SendChatMessage(msg, "INSTANCE_CHAT")
		end

		table.remove(reports);
	end
end

function addon.CreateGUI()
	local fRep = CreateFrame("Frame", addonName.."reportFrame", UIParent);
	addon.fRep = fRep;
	fRep:SetWidth(70);
	fRep:SetHeight(40);
	fRep:SetPoint("CENTER", UIParent, "CENTER");
	fRep:SetFrameStrata("HIGH");
	fRep.moving = false;
	fRep:SetMovable(true);

	local bRep = CreateFrame("Button", addonName.."reportFrameButton", fRep, "UIPanelButtonTemplate");
	bRep:SetAllPoints(fRep);
	bRep:SetPoint("CENTER", fRep, "CENTER");
	bRep:SetText("Report!");

	bRep:SetScript("OnMouseDown", function(self, button)
		if button == "RightButton" and not addon.fRep.isMoving then
			addon.fRep:StartMoving();
			addon.fRep.isMoving = true;
		end
	end)

	bRep:SetScript("OnMouseUp", function(self, button)
		if button == "RightButton" and addon.fRep.isMoving then
			addon.fRep:StopMovingOrSizing();
			addon.fRep.isMoving = false;
		elseif button == "LeftButton" then
			addon.HandleReport("target", true);
		elseif button == "MiddleButton" then
			addon.ShowGUI();
		end
	end)

	bRep:SetScript("OnMouseWheel", function(self, delta)
		addon.DeleteLastReport(delta);
	end)

	bRep:SetScript("OnHide", function(self)
		if addon.fRep.isMoving then
			addon.fRep:StopMovingOrSizing();
			addon.fRep.isMoving = false;
		end
	end)
end


function addon.ShowGUI()
	if addon.GUIShown == true then return end
	addon.GUIShown = true;
	local frame = AceGUI:Create("Frame");
	frame:SetTitle(L["Title"]);
	frame:SetStatusText(L["OneForAll"]);
	frame:SetWidth(600);
	frame:SetHeight(700);
	frame:SetCallback("OnClose", function(widget) 
		AceGUI:Release(widget);
		addon.GUIShown = false;
	end);
	frame:SetLayout("Flow");

	local memo = AceGUI:Create("MultiLineEditBox");
	memo:SetLabel(L["Copypaste"]);
	memo:DisableButton(true);
	memo:SetWidth(570);
	memo:SetHeight(620);
	memo:SetMaxLetters(0);
	local report = addon.MakeTextReport();
	memo:SetText(report);
	frame:AddChild(memo);
end

function addon.ShowHelp()
	local hlp = L.HelpText;
	for idx, value in ipairs(L["HelpText"]) do
		local str = value;
		print(str);
	end
	--print(hlp[1]);
end


function addon.CreateOptions()
	AceConfig:RegisterOptionsTable(L["AddonName"], options)
	AceConfigDialog:AddToBlizOptions(L["AddonName"])
end


